# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["RecordingTranscriptionListParams", "Filter", "FilterCreatedAt"]


class RecordingTranscriptionListParams(TypedDict, total=False):
    filter: Filter
    """Filter recording transcriptions by various attributes."""

    page_number: Annotated[int, PropertyInfo(alias="page[number]")]

    page_size: Annotated[int, PropertyInfo(alias="page[size]")]


class FilterCreatedAt(TypedDict, total=False):
    gte: str
    """Returns only transcriptions created later than or at given ISO 8601 datetime."""

    lte: str
    """Returns only transcriptions created earlier than or at given ISO 8601 datetime."""


class Filter(TypedDict, total=False):
    """Filter recording transcriptions by various attributes."""

    created_at: FilterCreatedAt

    recording_id: str
    """
    If present, transcriptions will be filtered to those associated with the given
    recording.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from .credit_account import (
    CreditAccountResource,
    AsyncCreditAccountResource,
    CreditAccountResourceWithRawResponse,
    AsyncCreditAccountResourceWithRawResponse,
    CreditAccountResourceWithStreamingResponse,
    AsyncCreditAccountResourceWithStreamingResponse,
)

__all__ = ["X402Resource", "AsyncX402Resource"]


class X402Resource(SyncAPIResource):
    @cached_property
    def credit_account(self) -> CreditAccountResource:
        """Operations for x402 cryptocurrency payment transactions.

        Fund your Telnyx account using USDC stablecoin payments via the x402 protocol.
        """
        return CreditAccountResource(self._client)

    @cached_property
    def with_raw_response(self) -> X402ResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return X402ResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> X402ResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return X402ResourceWithStreamingResponse(self)


class AsyncX402Resource(AsyncAPIResource):
    @cached_property
    def credit_account(self) -> AsyncCreditAccountResource:
        """Operations for x402 cryptocurrency payment transactions.

        Fund your Telnyx account using USDC stablecoin payments via the x402 protocol.
        """
        return AsyncCreditAccountResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncX402ResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncX402ResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncX402ResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncX402ResourceWithStreamingResponse(self)


class X402ResourceWithRawResponse:
    def __init__(self, x402: X402Resource) -> None:
        self._x402 = x402

    @cached_property
    def credit_account(self) -> CreditAccountResourceWithRawResponse:
        """Operations for x402 cryptocurrency payment transactions.

        Fund your Telnyx account using USDC stablecoin payments via the x402 protocol.
        """
        return CreditAccountResourceWithRawResponse(self._x402.credit_account)


class AsyncX402ResourceWithRawResponse:
    def __init__(self, x402: AsyncX402Resource) -> None:
        self._x402 = x402

    @cached_property
    def credit_account(self) -> AsyncCreditAccountResourceWithRawResponse:
        """Operations for x402 cryptocurrency payment transactions.

        Fund your Telnyx account using USDC stablecoin payments via the x402 protocol.
        """
        return AsyncCreditAccountResourceWithRawResponse(self._x402.credit_account)


class X402ResourceWithStreamingResponse:
    def __init__(self, x402: X402Resource) -> None:
        self._x402 = x402

    @cached_property
    def credit_account(self) -> CreditAccountResourceWithStreamingResponse:
        """Operations for x402 cryptocurrency payment transactions.

        Fund your Telnyx account using USDC stablecoin payments via the x402 protocol.
        """
        return CreditAccountResourceWithStreamingResponse(self._x402.credit_account)


class AsyncX402ResourceWithStreamingResponse:
    def __init__(self, x402: AsyncX402Resource) -> None:
        self._x402 = x402

    @cached_property
    def credit_account(self) -> AsyncCreditAccountResourceWithStreamingResponse:
        """Operations for x402 cryptocurrency payment transactions.

        Fund your Telnyx account using USDC stablecoin payments via the x402 protocol.
        """
        return AsyncCreditAccountResourceWithStreamingResponse(self._x402.credit_account)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["ActionDeleteResponse"]


class ActionDeleteResponse(BaseModel):
    status: Optional[Literal["ok"]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["TrafficPolicyProfileListParams"]


class TrafficPolicyProfileListParams(TypedDict, total=False):
    filter_service: Annotated[str, PropertyInfo(alias="filter[service]")]
    """Filter by service ID."""

    filter_type: Annotated[Literal["whitelist", "blacklist", "throttling"], PropertyInfo(alias="filter[type]")]
    """Filter by traffic policy profile type."""

    page_number: Annotated[int, PropertyInfo(alias="page[number]")]
    """The page number to load."""

    page_size: Annotated[int, PropertyInfo(alias="page[size]")]
    """The size of the page."""

    sort: Literal["service", "-service", "type", "-type"]
    """Sorts traffic policy profiles by the given field.

    Defaults to ascending order unless field is prefixed with a minus sign.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from pydantic import Field as FieldInfo

from ..._models import BaseModel
from ..rcs_to_item import RcsToItem
from ..rcs_agent_message import RcsAgentMessage

__all__ = ["RcSendResponse", "Data", "DataFrom"]


class DataFrom(BaseModel):
    agent_id: Optional[str] = None
    """agent ID"""

    agent_name: Optional[str] = None

    carrier: Optional[str] = None


class Data(BaseModel):
    id: Optional[str] = None
    """message ID"""

    body: Optional[RcsAgentMessage] = None

    direction: Optional[str] = None

    encoding: Optional[str] = None

    from_: Optional[DataFrom] = FieldInfo(alias="from", default=None)

    messaging_profile_id: Optional[str] = None

    organization_id: Optional[str] = None

    received_at: Optional[datetime] = None

    record_type: Optional[str] = None

    to: Optional[List[RcsToItem]] = None

    type: Optional[str] = None

    wait_seconds: Optional[float] = None
    """
    Seconds the message is queued due to rate limiting before being sent to the
    carrier. Represents the maximum wait across all applicable rate limits (account,
    carrier, campaign). 0.0 = no queuing delay.
    """


class RcSendResponse(BaseModel):
    data: Optional[Data] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.types.storage.buckets import (
    SslCertificateCreateResponse,
    SslCertificateDeleteResponse,
    SslCertificateRetrieveResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestSslCertificate:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create(self, client: Telnyx) -> None:
        ssl_certificate = client.storage.buckets.ssl_certificate.create(
            bucket_name="",
        )
        assert_matches_type(SslCertificateCreateResponse, ssl_certificate, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Telnyx) -> None:
        ssl_certificate = client.storage.buckets.ssl_certificate.create(
            bucket_name="",
            certificate=b"Example data",
            private_key=b"Example data",
        )
        assert_matches_type(SslCertificateCreateResponse, ssl_certificate, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Telnyx) -> None:
        response = client.storage.buckets.ssl_certificate.with_raw_response.create(
            bucket_name="",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        ssl_certificate = response.parse()
        assert_matches_type(SslCertificateCreateResponse, ssl_certificate, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Telnyx) -> None:
        with client.storage.buckets.ssl_certificate.with_streaming_response.create(
            bucket_name="",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            ssl_certificate = response.parse()
            assert_matches_type(SslCertificateCreateResponse, ssl_certificate, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_create(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `bucket_name` but received ''"):
            client.storage.buckets.ssl_certificate.with_raw_response.create(
                bucket_name="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Telnyx) -> None:
        ssl_certificate = client.storage.buckets.ssl_certificate.retrieve(
            "",
        )
        assert_matches_type(SslCertificateRetrieveResponse, ssl_certificate, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Telnyx) -> None:
        response = client.storage.buckets.ssl_certificate.with_raw_response.retrieve(
            "",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        ssl_certificate = response.parse()
        assert_matches_type(SslCertificateRetrieveResponse, ssl_certificate, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Telnyx) -> None:
        with client.storage.buckets.ssl_certificate.with_streaming_response.retrieve(
            "",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            ssl_certificate = response.parse()
            assert_matches_type(SslCertificateRetrieveResponse, ssl_certificate, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `bucket_name` but received ''"):
            client.storage.buckets.ssl_certificate.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_delete(self, client: Telnyx) -> None:
        ssl_certificate = client.storage.buckets.ssl_certificate.delete(
            "",
        )
        assert_matches_type(SslCertificateDeleteResponse, ssl_certificate, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Telnyx) -> None:
        response = client.storage.buckets.ssl_certificate.with_raw_response.delete(
            "",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        ssl_certificate = response.parse()
        assert_matches_type(SslCertificateDeleteResponse, ssl_certificate, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Telnyx) -> None:
        with client.storage.buckets.ssl_certificate.with_streaming_response.delete(
            "",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            ssl_certificate = response.parse()
            assert_matches_type(SslCertificateDeleteResponse, ssl_certificate, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `bucket_name` but received ''"):
            client.storage.buckets.ssl_certificate.with_raw_response.delete(
                "",
            )


class TestAsyncSslCertificate:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncTelnyx) -> None:
        ssl_certificate = await async_client.storage.buckets.ssl_certificate.create(
            bucket_name="",
        )
        assert_matches_type(SslCertificateCreateResponse, ssl_certificate, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncTelnyx) -> None:
        ssl_certificate = await async_client.storage.buckets.ssl_certificate.create(
            bucket_name="",
            certificate=b"Example data",
            private_key=b"Example data",
        )
        assert_matches_type(SslCertificateCreateResponse, ssl_certificate, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.storage.buckets.ssl_certificate.with_raw_response.create(
            bucket_name="",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        ssl_certificate = await response.parse()
        assert_matches_type(SslCertificateCreateResponse, ssl_certificate, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncTelnyx) -> None:
        async with async_client.storage.buckets.ssl_certificate.with_streaming_response.create(
            bucket_name="",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            ssl_certificate = await response.parse()
            assert_matches_type(SslCertificateCreateResponse, ssl_certificate, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_create(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `bucket_name` but received ''"):
            await async_client.storage.buckets.ssl_certificate.with_raw_response.create(
                bucket_name="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncTelnyx) -> None:
        ssl_certificate = await async_client.storage.buckets.ssl_certificate.retrieve(
            "",
        )
        assert_matches_type(SslCertificateRetrieveResponse, ssl_certificate, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.storage.buckets.ssl_certificate.with_raw_response.retrieve(
            "",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        ssl_certificate = await response.parse()
        assert_matches_type(SslCertificateRetrieveResponse, ssl_certificate, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        async with async_client.storage.buckets.ssl_certificate.with_streaming_response.retrieve(
            "",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            ssl_certificate = await response.parse()
            assert_matches_type(SslCertificateRetrieveResponse, ssl_certificate, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `bucket_name` but received ''"):
            await async_client.storage.buckets.ssl_certificate.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncTelnyx) -> None:
        ssl_certificate = await async_client.storage.buckets.ssl_certificate.delete(
            "",
        )
        assert_matches_type(SslCertificateDeleteResponse, ssl_certificate, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.storage.buckets.ssl_certificate.with_raw_response.delete(
            "",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        ssl_certificate = await response.parse()
        assert_matches_type(SslCertificateDeleteResponse, ssl_certificate, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncTelnyx) -> None:
        async with async_client.storage.buckets.ssl_certificate.with_streaming_response.delete(
            "",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            ssl_certificate = await response.parse()
            assert_matches_type(SslCertificateDeleteResponse, ssl_certificate, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `bucket_name` but received ''"):
            await async_client.storage.buckets.ssl_certificate.with_raw_response.delete(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal, TypedDict

from .._types import SequenceNotStr

__all__ = ["TrafficPolicyProfileUpdateParams"]


class TrafficPolicyProfileUpdateParams(TypedDict, total=False):
    domains: SequenceNotStr[str]
    """Array of domain names."""

    ip_ranges: SequenceNotStr[str]
    """Array of IP ranges in CIDR notation."""

    limit_bw_kbps: Optional[Literal[512, 1024]]
    """Bandwidth limit in kbps. Must be 512 or 1024, or null to remove."""

    services: SequenceNotStr[str]
    """Array of PCEF service IDs to include in the profile."""

    type: Literal["whitelist", "blacklist", "throttling"]
    """The type of the traffic policy profile."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["ActionJoinAIAssistantParams", "Participant"]


class ActionJoinAIAssistantParams(TypedDict, total=False):
    conversation_id: Required[str]
    """The ID of the AI assistant conversation to join."""

    participant: Required[Participant]

    client_state: str
    """Use this field to add state to every subsequent webhook.

    It must be a valid Base-64 encoded string.
    """

    command_id: str
    """Use this field to avoid duplicate commands.

    Telnyx will ignore any command with the same `command_id` for the same
    `call_control_id`.
    """


class Participant(TypedDict, total=False):
    id: Required[str]
    """The call_control_id of the participant to add to the conversation."""

    role: Required[Literal["user"]]
    """The role of the participant in the conversation."""

    name: str
    """Display name for the participant."""

    on_hangup: Literal["continue_conversation", "end_conversation"]
    """Determines what happens to the conversation when this participant hangs up."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["NumberOrderPhoneNumberListParams", "Filter"]


class NumberOrderPhoneNumberListParams(TypedDict, total=False):
    filter: Filter
    """Consolidated filter parameter (deepObject style).

    Originally: filter[country_code]
    """


class Filter(TypedDict, total=False):
    """Consolidated filter parameter (deepObject style).

    Originally: filter[country_code]
    """

    country_code: str
    """Country code of the order phone number."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import json
import logging
from types import TracebackType
from typing import TYPE_CHECKING, Any, Iterator, cast
from typing_extensions import Literal, AsyncIterator

import httpx

from ..types import speech_to_text_transcribe_params
from .._types import Body, Query, Headers, NoneType, NotGiven, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._models import construct_type_unchecked
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    async_to_raw_response_wrapper,
)
from .._exceptions import TelnyxError
from .._base_client import _merge_mappings, make_request_options
from ..types.stt_server_event import SttServerEvent
from ..types.websocket_connection_options import WebSocketConnectionOptions

if TYPE_CHECKING:
    from websockets.sync.client import ClientConnection as WebsocketConnection
    from websockets.asyncio.client import ClientConnection as AsyncWebsocketConnection

    from .._client import Telnyx, AsyncTelnyx

__all__ = ["SpeechToTextResource", "AsyncSpeechToTextResource"]

log: logging.Logger = logging.getLogger(__name__)

# Type alias for binary audio data
SttClientEvent = bytes


class SpeechToTextResource(SyncAPIResource):
    """Speech to text streaming command operations via WebSocket"""

    @cached_property
    def with_raw_response(self) -> SpeechToTextResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return SpeechToTextResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SpeechToTextResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return SpeechToTextResourceWithStreamingResponse(self)

    def transcribe(
        self,
        *,
        input_format: Literal["mp3", "wav"],
        transcription_engine: Literal["Azure", "Deepgram", "Google", "Telnyx"],
        endpointing: int | NotGiven = not_given,
        interim_results: bool | NotGiven = not_given,
        keyterm: str | NotGiven = not_given,
        keywords: str | NotGiven = not_given,
        language: str | NotGiven = not_given,
        model: Literal[
            "fast",
            "deepgram/nova-2",
            "deepgram/nova-3",
            "latest_long",
            "latest_short",
            "command_and_search",
            "phone_call",
            "video",
            "default",
            "medical_conversation",
            "medical_dictation",
            "openai/whisper-tiny",
            "openai/whisper-large-v3-turbo",
        ]
        | NotGiven = not_given,
        redact: str | NotGiven = not_given,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Open a WebSocket connection to stream audio and receive transcriptions in
        real-time. Authentication is provided via the standard
        `Authorization: Bearer <API_KEY>` header.

        Supported engines: `Azure`, `Deepgram`, `Google`, `Telnyx`.

        **Connection flow:**

        1. Open WebSocket with query parameters specifying engine, input format, and
           language.
        2. Send binary audio frames (mp3/wav format).
        3. Receive JSON transcript frames with `transcript`, `is_final`, and
           `confidence` fields.
        4. Close connection when done.

        Args:
          input_format: The format of input audio stream.

          transcription_engine: The transcription engine to use for processing the audio stream.

          endpointing: Silence duration (in milliseconds) that triggers end-of-speech detection.
              When set, the engine uses this value to determine when a speaker has stopped
              talking. Not all engines support this parameter.

          interim_results: Whether to receive interim transcription results.

          keyterm: A key term to boost in the transcription. The engine will be more likely to
              recognize this term. Can be specified multiple times for multiple terms.

          keywords: Comma-separated list of keywords to boost in the transcription. The engine will
              prioritize recognition of these words.

          language: The language spoken in the audio stream.

          model: The specific model to use within the selected transcription engine.

          redact: Enable redaction of sensitive information (e.g., PCI data, SSN) from
              transcription results. Supported values depend on the transcription engine.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._get(
            "/speech-to-text/transcription",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "input_format": input_format,
                        "transcription_engine": transcription_engine,
                        "endpointing": endpointing,
                        "interim_results": interim_results,
                        "keyterm": keyterm,
                        "keywords": keywords,
                        "language": language,
                        "model": model,
                        "redact": redact,
                    },
                    speech_to_text_transcribe_params.SpeechToTextTranscribeParams,
                ),
            ),
            cast_to=NoneType,
        )

    def stream(
        self,
        *,
        transcription_engine: Literal["Azure", "Deepgram", "Google", "Telnyx"],
        input_format: Literal["mp3", "wav", "raw"] | None = None,
        language: str | None = None,
        interim_results: bool | None = None,
        model: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        websocket_connection_options: WebSocketConnectionOptions = {},
    ) -> SpeechToTextResourceConnectionManager:
        """
        Create a WebSocket connection for real-time speech-to-text transcription.

        Args:
          transcription_engine: The transcription engine to use for processing the audio stream.
              Supported engines: `Azure`, `Deepgram`, `Google`, `Telnyx`.

          input_format: The format of the input audio stream. Defaults to `wav`.

          language: The language code for transcription (e.g., 'en-US', 'es-ES').

          interim_results: Whether to return interim (partial) transcription results.

          model: The model to use for transcription (engine-specific).

          extra_headers: Additional headers to send with the request.

          extra_query: Additional query parameters to send with the request.

          websocket_connection_options: Additional options for the WebSocket connection.

        Returns:
          A context manager that yields a `SpeechToTextResourceConnection` when entered.

        Example:
            ```python
            with client.speech_to_text.stream(transcription_engine="Telnyx") as connection:
                connection.send(audio_data)
                for event in connection:
                    if event.type == "transcript":
                        print(event.transcript)
            ```
        """
        return SpeechToTextResourceConnectionManager(
            client=self._client,
            transcription_engine=transcription_engine,
            input_format=input_format,
            language=language,
            interim_results=interim_results,
            model=model,
            extra_query=extra_query or {},
            extra_headers=extra_headers or {},
            websocket_connection_options=websocket_connection_options,
        )


class AsyncSpeechToTextResource(AsyncAPIResource):
    """Speech to text streaming command operations via WebSocket"""

    @cached_property
    def with_raw_response(self) -> AsyncSpeechToTextResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSpeechToTextResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSpeechToTextResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncSpeechToTextResourceWithStreamingResponse(self)

    async def transcribe(
        self,
        *,
        input_format: Literal["mp3", "wav"],
        transcription_engine: Literal["Azure", "Deepgram", "Google", "Telnyx"],
        endpointing: int | NotGiven = not_given,
        interim_results: bool | NotGiven = not_given,
        keyterm: str | NotGiven = not_given,
        keywords: str | NotGiven = not_given,
        language: str | NotGiven = not_given,
        model: Literal[
            "fast",
            "deepgram/nova-2",
            "deepgram/nova-3",
            "latest_long",
            "latest_short",
            "command_and_search",
            "phone_call",
            "video",
            "default",
            "medical_conversation",
            "medical_dictation",
            "openai/whisper-tiny",
            "openai/whisper-large-v3-turbo",
        ]
        | NotGiven = not_given,
        redact: str | NotGiven = not_given,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Open a WebSocket connection to stream audio and receive transcriptions in
        real-time. Authentication is provided via the standard
        `Authorization: Bearer <API_KEY>` header.

        Supported engines: `Azure`, `Deepgram`, `Google`, `Telnyx`.

        **Connection flow:**

        1. Open WebSocket with query parameters specifying engine, input format, and
           language.
        2. Send binary audio frames (mp3/wav format).
        3. Receive JSON transcript frames with `transcript`, `is_final`, and
           `confidence` fields.
        4. Close connection when done.

        Args:
          input_format: The format of input audio stream.

          transcription_engine: The transcription engine to use for processing the audio stream.

          endpointing: Silence duration (in milliseconds) that triggers end-of-speech detection.
              When set, the engine uses this value to determine when a speaker has stopped
              talking. Not all engines support this parameter.

          interim_results: Whether to receive interim transcription results.

          keyterm: A key term to boost in the transcription. The engine will be more likely to
              recognize this term. Can be specified multiple times for multiple terms.

          keywords: Comma-separated list of keywords to boost in the transcription. The engine will
              prioritize recognition of these words.

          language: The language spoken in the audio stream.

          model: The specific model to use within the selected transcription engine.

          redact: Enable redaction of sensitive information (e.g., PCI data, SSN) from
              transcription results. Supported values depend on the transcription engine.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._get(
            "/speech-to-text/transcription",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "input_format": input_format,
                        "transcription_engine": transcription_engine,
                        "endpointing": endpointing,
                        "interim_results": interim_results,
                        "keyterm": keyterm,
                        "keywords": keywords,
                        "language": language,
                        "model": model,
                        "redact": redact,
                    },
                    speech_to_text_transcribe_params.SpeechToTextTranscribeParams,
                ),
            ),
            cast_to=NoneType,
        )

    def stream(
        self,
        *,
        transcription_engine: Literal["Azure", "Deepgram", "Google", "Telnyx"],
        input_format: Literal["mp3", "wav", "raw"] | None = None,
        language: str | None = None,
        interim_results: bool | None = None,
        model: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        websocket_connection_options: WebSocketConnectionOptions = {},
    ) -> AsyncSpeechToTextResourceConnectionManager:
        """
        Create a WebSocket connection for real-time speech-to-text transcription.

        Args:
          transcription_engine: The transcription engine to use for processing the audio stream.
              Supported engines: `Azure`, `Deepgram`, `Google`, `Telnyx`.

          input_format: The format of the input audio stream. Defaults to `wav`.

          language: The language code for transcription (e.g., 'en-US', 'es-ES').

          interim_results: Whether to return interim (partial) transcription results.

          model: The model to use for transcription (engine-specific).

          extra_headers: Additional headers to send with the request.

          extra_query: Additional query parameters to send with the request.

          websocket_connection_options: Additional options for the WebSocket connection.

        Returns:
          A context manager that yields an `AsyncSpeechToTextResourceConnection` when entered.

        Example:
            ```python
            async with client.speech_to_text.stream(transcription_engine="Telnyx") as connection:
                await connection.send(audio_data)
                async for event in connection:
                    if event.type == "transcript":
                        print(event.transcript)
            ```
        """
        return AsyncSpeechToTextResourceConnectionManager(
            client=self._client,
            transcription_engine=transcription_engine,
            input_format=input_format,
            language=language,
            interim_results=interim_results,
            model=model,
            extra_query=extra_query or {},
            extra_headers=extra_headers or {},
            websocket_connection_options=websocket_connection_options,
        )


class SpeechToTextResourceWithRawResponse:
    def __init__(self, speech_to_text: SpeechToTextResource) -> None:
        self._speech_to_text = speech_to_text

        self.transcribe = to_raw_response_wrapper(
            speech_to_text.transcribe,
        )


class AsyncSpeechToTextResourceWithRawResponse:
    def __init__(self, speech_to_text: AsyncSpeechToTextResource) -> None:
        self._speech_to_text = speech_to_text

        self.transcribe = async_to_raw_response_wrapper(
            speech_to_text.transcribe,
        )


class AsyncSpeechToTextResourceConnection:
    """Represents a live WebSocket connection to the SpeechToText API"""

    _connection: AsyncWebsocketConnection

    def __init__(self, connection: AsyncWebsocketConnection) -> None:
        self._connection = connection

    async def __aiter__(self) -> AsyncIterator[SttServerEvent]:
        """
        An infinite-iterator that will continue to yield events until
        the connection is closed.
        """
        from websockets.exceptions import ConnectionClosedOK

        try:
            while True:
                yield await self.recv()
        except ConnectionClosedOK:
            return

    async def recv(self) -> SttServerEvent:
        """
        Receive the next message from the connection and parses it into a `SttServerEvent` object.

        Canceling this method is safe. There's no risk of losing data.
        """
        return self.parse_event(await self.recv_bytes())

    async def recv_bytes(self) -> bytes:
        """Receive the next message from the connection as raw bytes.

        Canceling this method is safe. There's no risk of losing data.

        If you want to parse the message into a `SttServerEvent` object like `.recv()` does,
        then you can call `.parse_event(data)`.
        """
        message = await self._connection.recv(decode=False)
        log.debug(f"Received websocket message: %s", message)
        return message

    async def send(self, data: SttClientEvent) -> None:
        """Send binary audio data to the server for transcription.

        Args:
            data: Raw audio bytes (mp3, wav, or raw format depending on input_format parameter).
        """
        await self._connection.send(data)

    async def close(self, *, code: int = 1000, reason: str = "") -> None:
        await self._connection.close(code=code, reason=reason)

    def parse_event(self, data: str | bytes) -> SttServerEvent:
        """
        Converts a raw `str` or `bytes` message into a `SttServerEvent` object.

        This is helpful if you're using `.recv_bytes()`.
        """
        return cast(SttServerEvent, construct_type_unchecked(value=json.loads(data), type_=cast(Any, SttServerEvent)))


class AsyncSpeechToTextResourceConnectionManager:
    """
    Context manager over a `AsyncSpeechToTextResourceConnection` that is returned by `speech_to_text.stream()`

    This context manager ensures that the connection will be closed when it exits.

    ---

    Note that if your application doesn't work well with the context manager approach then you
    can call the `.enter()` method directly to initiate a connection.

    **Warning**: You must remember to close the connection with `.close()`.

    ```py
    connection = await client.speech_to_text.stream(...).enter()
    # ...
    await connection.close()
    ```
    """

    def __init__(
        self,
        *,
        client: AsyncTelnyx,
        transcription_engine: Literal["Azure", "Deepgram", "Google", "Telnyx"],
        input_format: Literal["mp3", "wav", "raw"] | None,
        language: str | None,
        interim_results: bool | None,
        model: str | None,
        extra_query: Query,
        extra_headers: Headers,
        websocket_connection_options: WebSocketConnectionOptions,
    ) -> None:
        self.__client = client
        self.__connection: AsyncSpeechToTextResourceConnection | None = None
        self.__transcription_engine = transcription_engine
        self.__input_format = input_format
        self.__language = language
        self.__interim_results = interim_results
        self.__model = model
        self.__extra_query = extra_query
        self.__extra_headers = extra_headers
        self.__websocket_connection_options = websocket_connection_options

    async def __aenter__(self) -> AsyncSpeechToTextResourceConnection:
        """
        👋 If your application doesn't work well with the context manager approach then you
        can call this method directly to initiate a connection.

        **Warning**: You must remember to close the connection with `.close()`.

        ```py
        connection = await client.speech_to_text.stream(...).enter()
        # ...
        await connection.close()
        ```
        """
        try:
            from websockets.asyncio.client import connect
        except ImportError as exc:
            raise TelnyxError("You need to install `telnyx[websockets]` to use this method") from exc

        url = self._prepare_url().copy_with(
            params={
                **self.__client.base_url.params,
                **self.__extra_query,
            },
        )
        log.debug("Connecting to %s", url)
        if self.__websocket_connection_options:
            log.debug("Connection options: %s", self.__websocket_connection_options)

        self.__connection = AsyncSpeechToTextResourceConnection(
            await connect(
                str(url),
                user_agent_header=self.__client.user_agent,
                additional_headers=_merge_mappings(
                    {
                        **self.__client.auth_headers,
                    },
                    self.__extra_headers,
                ),
                **self.__websocket_connection_options,
            )
        )

        return self.__connection

    enter = __aenter__

    def _prepare_url(self) -> httpx.URL:
        if self.__client.websocket_base_url is not None:
            base_url = httpx.URL(self.__client.websocket_base_url)
        else:
            base_url = self.__client._base_url.copy_with(scheme="wss")

        # Build query parameters
        params: dict[str, str] = {"transcription_engine": self.__transcription_engine}
        if self.__input_format is not None:
            params["input_format"] = self.__input_format
        if self.__language is not None:
            params["language"] = self.__language
        if self.__interim_results is not None:
            params["interim_results"] = str(self.__interim_results).lower()
        if self.__model is not None:
            params["model"] = self.__model

        merge_raw_path = base_url.raw_path.rstrip(b"/") + b"/speech-to-text/transcription"
        return base_url.copy_with(raw_path=merge_raw_path, params=params)

    async def __aexit__(
        self, exc_type: type[BaseException] | None, exc: BaseException | None, exc_tb: TracebackType | None
    ) -> None:
        if self.__connection is not None:
            await self.__connection.close()


class SpeechToTextResourceConnection:
    """Represents a live WebSocket connection to the SpeechToText API"""

    _connection: WebsocketConnection

    def __init__(self, connection: WebsocketConnection) -> None:
        self._connection = connection

    def __iter__(self) -> Iterator[SttServerEvent]:
        """
        An infinite-iterator that will continue to yield events until
        the connection is closed.
        """
        from websockets.exceptions import ConnectionClosedOK

        try:
            while True:
                yield self.recv()
        except ConnectionClosedOK:
            return

    def recv(self) -> SttServerEvent:
        """
        Receive the next message from the connection and parses it into a `SttServerEvent` object.

        Canceling this method is safe. There's no risk of losing data.
        """
        return self.parse_event(self.recv_bytes())

    def recv_bytes(self) -> bytes:
        """Receive the next message from the connection as raw bytes.

        Canceling this method is safe. There's no risk of losing data.

        If you want to parse the message into a `SttServerEvent` object like `.recv()` does,
        then you can call `.parse_event(data)`.
        """
        message = self._connection.recv(decode=False)
        log.debug(f"Received websocket message: %s", message)
        return message

    def send(self, data: SttClientEvent) -> None:
        """Send binary audio data to the server for transcription.

        Args:
            data: Raw audio bytes (mp3, wav, or raw format depending on input_format parameter).
        """
        self._connection.send(data)

    def close(self, *, code: int = 1000, reason: str = "") -> None:
        self._connection.close(code=code, reason=reason)

    def parse_event(self, data: str | bytes) -> SttServerEvent:
        """
        Converts a raw `str` or `bytes` message into a `SttServerEvent` object.

        This is helpful if you're using `.recv_bytes()`.
        """
        return cast(SttServerEvent, construct_type_unchecked(value=json.loads(data), type_=cast(Any, SttServerEvent)))


class SpeechToTextResourceConnectionManager:
    """
    Context manager over a `SpeechToTextResourceConnection` that is returned by `speech_to_text.stream()`

    This context manager ensures that the connection will be closed when it exits.

    ---

    Note that if your application doesn't work well with the context manager approach then you
    can call the `.enter()` method directly to initiate a connection.

    **Warning**: You must remember to close the connection with `.close()`.

    ```py
    connection = client.speech_to_text.stream(...).enter()
    # ...
    connection.close()
    ```
    """

    def __init__(
        self,
        *,
        client: Telnyx,
        transcription_engine: Literal["Azure", "Deepgram", "Google", "Telnyx"],
        input_format: Literal["mp3", "wav", "raw"] | None,
        language: str | None,
        interim_results: bool | None,
        model: str | None,
        extra_query: Query,
        extra_headers: Headers,
        websocket_connection_options: WebSocketConnectionOptions,
    ) -> None:
        self.__client = client
        self.__connection: SpeechToTextResourceConnection | None = None
        self.__transcription_engine = transcription_engine
        self.__input_format = input_format
        self.__language = language
        self.__interim_results = interim_results
        self.__model = model
        self.__extra_query = extra_query
        self.__extra_headers = extra_headers
        self.__websocket_connection_options = websocket_connection_options

    def __enter__(self) -> SpeechToTextResourceConnection:
        """
        👋 If your application doesn't work well with the context manager approach then you
        can call this method directly to initiate a connection.

        **Warning**: You must remember to close the connection with `.close()`.

        ```py
        connection = client.speech_to_text.stream(...).enter()
        # ...
        connection.close()
        ```
        """
        try:
            from websockets.sync.client import connect
        except ImportError as exc:
            raise TelnyxError("You need to install `telnyx[websockets]` to use this method") from exc

        url = self._prepare_url().copy_with(
            params={
                **self.__client.base_url.params,
                **self.__extra_query,
            },
        )
        log.debug("Connecting to %s", url)
        if self.__websocket_connection_options:
            log.debug("Connection options: %s", self.__websocket_connection_options)

        self.__connection = SpeechToTextResourceConnection(
            connect(
                str(url),
                user_agent_header=self.__client.user_agent,
                additional_headers=_merge_mappings(
                    {
                        **self.__client.auth_headers,
                    },
                    self.__extra_headers,
                ),
                **self.__websocket_connection_options,
            )
        )

        return self.__connection

    enter = __enter__

    def _prepare_url(self) -> httpx.URL:
        if self.__client.websocket_base_url is not None:
            base_url = httpx.URL(self.__client.websocket_base_url)
        else:
            base_url = self.__client._base_url.copy_with(scheme="wss")

        # Build query parameters
        params: dict[str, str] = {"transcription_engine": self.__transcription_engine}
        if self.__input_format is not None:
            params["input_format"] = self.__input_format
        if self.__language is not None:
            params["language"] = self.__language
        if self.__interim_results is not None:
            params["interim_results"] = str(self.__interim_results).lower()
        if self.__model is not None:
            params["model"] = self.__model

        merge_raw_path = base_url.raw_path.rstrip(b"/") + b"/speech-to-text/transcription"
        return base_url.copy_with(raw_path=merge_raw_path, params=params)

    def __exit__(
        self, exc_type: type[BaseException] | None, exc: BaseException | None, exc_tb: TracebackType | None
    ) -> None:
        if self.__connection is not None:
            self.__connection.close()


class SpeechToTextResourceWithStreamingResponse:
    def __init__(self, speech_to_text: SpeechToTextResource) -> None:
        self._speech_to_text = speech_to_text


class AsyncSpeechToTextResourceWithStreamingResponse:
    def __init__(self, speech_to_text: AsyncSpeechToTextResource) -> None:
        self._speech_to_text = speech_to_text



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union, Optional
from typing_extensions import Literal, Annotated, TypeAlias

from .._utils import PropertyInfo
from .._models import BaseModel

__all__ = ["SttServerEvent", "TranscriptFrame", "SttErrorFrame"]


class TranscriptFrame(BaseModel):
    """Server-to-client frame containing a transcription result."""

    type: Optional[Literal["transcript"]] = None
    """Frame type identifier."""

    transcript: Optional[str] = None
    """The transcribed text from the audio."""

    is_final: Optional[bool] = None
    """Whether this is a final transcription result.

    When `false`, this is an interim result that may be refined.
    """

    confidence: Optional[float] = None
    """Confidence score of the transcription, ranging from 0 to 1."""


class SttErrorFrame(BaseModel):
    """Server-to-client frame indicating an error during transcription.

    The connection is closed shortly after.
    """

    type: Optional[Literal["error"]] = None
    """Frame type identifier."""

    error: Optional[str] = None
    """Error message describing what went wrong."""


SttServerEvent: TypeAlias = Annotated[Union[TranscriptFrame, SttErrorFrame], PropertyInfo(discriminator="type")]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["TrafficPolicyProfileDeleteResponse", "Data"]


class Data(BaseModel):
    id: Optional[str] = None
    """Identifies the resource."""


class TrafficPolicyProfileDeleteResponse(BaseModel):
    data: Optional[Data] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .action_delete_params import ActionDeleteParams as ActionDeleteParams
from .action_delete_response import ActionDeleteResponse as ActionDeleteResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["TrafficPolicyProfileListServicesResponse"]


class TrafficPolicyProfileListServicesResponse(BaseModel):
    id: Optional[str] = None
    """The service identifier."""

    group: Optional[str] = None
    """The group the service belongs to."""

    name: Optional[str] = None
    """The name of the service."""

    resource_type: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal

import httpx

from ..types import (
    traffic_policy_profile_list_params,
    traffic_policy_profile_create_params,
    traffic_policy_profile_update_params,
    traffic_policy_profile_list_services_params,
)
from .._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from .._utils import path_template, maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from .._base_client import AsyncPaginator, make_request_options
from ..types.traffic_policy_profile import TrafficPolicyProfile
from ..types.traffic_policy_profile_create_response import TrafficPolicyProfileCreateResponse
from ..types.traffic_policy_profile_delete_response import TrafficPolicyProfileDeleteResponse
from ..types.traffic_policy_profile_update_response import TrafficPolicyProfileUpdateResponse
from ..types.traffic_policy_profile_retrieve_response import TrafficPolicyProfileRetrieveResponse
from ..types.traffic_policy_profile_list_services_response import TrafficPolicyProfileListServicesResponse

__all__ = ["TrafficPolicyProfilesResource", "AsyncTrafficPolicyProfilesResource"]


class TrafficPolicyProfilesResource(SyncAPIResource):
    """Traffic Policy Profiles operations"""

    @cached_property
    def with_raw_response(self) -> TrafficPolicyProfilesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return TrafficPolicyProfilesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TrafficPolicyProfilesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return TrafficPolicyProfilesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        type: Literal["whitelist", "blacklist"],
        domains: SequenceNotStr[str] | Omit = omit,
        ip_ranges: SequenceNotStr[str] | Omit = omit,
        limit_bw_kbps: Literal[512, 1024] | Omit = omit,
        services: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TrafficPolicyProfileCreateResponse:
        """Create a new traffic policy profile.

        At least one of `services`, `ip_ranges`, or
        `domains` must be provided.

        Args:
          type: The type of the traffic policy profile.

          domains: Array of domain names.

          ip_ranges: Array of IP ranges in CIDR notation.

          limit_bw_kbps: Bandwidth limit in kbps. Must be 512 or 1024.

          services: Array of PCEF service IDs to include in the profile.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/traffic_policy_profiles",
            body=maybe_transform(
                {
                    "type": type,
                    "domains": domains,
                    "ip_ranges": ip_ranges,
                    "limit_bw_kbps": limit_bw_kbps,
                    "services": services,
                },
                traffic_policy_profile_create_params.TrafficPolicyProfileCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TrafficPolicyProfileCreateResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TrafficPolicyProfileRetrieveResponse:
        """
        Returns the details regarding a specific traffic policy profile.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            path_template("/traffic_policy_profiles/{id}", id=id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TrafficPolicyProfileRetrieveResponse,
        )

    def update(
        self,
        id: str,
        *,
        domains: SequenceNotStr[str] | Omit = omit,
        ip_ranges: SequenceNotStr[str] | Omit = omit,
        limit_bw_kbps: Optional[Literal[512, 1024]] | Omit = omit,
        services: SequenceNotStr[str] | Omit = omit,
        type: Literal["whitelist", "blacklist", "throttling"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TrafficPolicyProfileUpdateResponse:
        """
        Updates a traffic policy profile.

        Args:
          domains: Array of domain names.

          ip_ranges: Array of IP ranges in CIDR notation.

          limit_bw_kbps: Bandwidth limit in kbps. Must be 512 or 1024, or null to remove.

          services: Array of PCEF service IDs to include in the profile.

          type: The type of the traffic policy profile.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._patch(
            path_template("/traffic_policy_profiles/{id}", id=id),
            body=maybe_transform(
                {
                    "domains": domains,
                    "ip_ranges": ip_ranges,
                    "limit_bw_kbps": limit_bw_kbps,
                    "services": services,
                    "type": type,
                },
                traffic_policy_profile_update_params.TrafficPolicyProfileUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TrafficPolicyProfileUpdateResponse,
        )

    def list(
        self,
        *,
        filter_service: str | Omit = omit,
        filter_type: Literal["whitelist", "blacklist", "throttling"] | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        sort: Literal["service", "-service", "type", "-type"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[TrafficPolicyProfile]:
        """
        Get all traffic policy profiles belonging to the user that match the given
        filters.

        Args:
          filter_service: Filter by service ID.

          filter_type: Filter by traffic policy profile type.

          page_number: The page number to load.

          page_size: The size of the page.

          sort: Sorts traffic policy profiles by the given field. Defaults to ascending order
              unless field is prefixed with a minus sign.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/traffic_policy_profiles",
            page=SyncDefaultFlatPagination[TrafficPolicyProfile],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter_service": filter_service,
                        "filter_type": filter_type,
                        "page_number": page_number,
                        "page_size": page_size,
                        "sort": sort,
                    },
                    traffic_policy_profile_list_params.TrafficPolicyProfileListParams,
                ),
            ),
            model=TrafficPolicyProfile,
        )

    def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TrafficPolicyProfileDeleteResponse:
        """
        Deletes the traffic policy profile.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            path_template("/traffic_policy_profiles/{id}", id=id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TrafficPolicyProfileDeleteResponse,
        )

    def list_services(
        self,
        *,
        filter_group: str | Omit = omit,
        filter_name: str | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[TrafficPolicyProfileListServicesResponse]:
        """
        Get all available PCEF services that can be used in traffic policy profiles.

        Args:
          filter_group: Filter services by group.

          filter_name: Filter services by name.

          page_number: The page number to load.

          page_size: The size of the page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/traffic_policy_profiles/services",
            page=SyncDefaultFlatPagination[TrafficPolicyProfileListServicesResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter_group": filter_group,
                        "filter_name": filter_name,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    traffic_policy_profile_list_services_params.TrafficPolicyProfileListServicesParams,
                ),
            ),
            model=TrafficPolicyProfileListServicesResponse,
        )


class AsyncTrafficPolicyProfilesResource(AsyncAPIResource):
    """Traffic Policy Profiles operations"""

    @cached_property
    def with_raw_response(self) -> AsyncTrafficPolicyProfilesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTrafficPolicyProfilesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTrafficPolicyProfilesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncTrafficPolicyProfilesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        type: Literal["whitelist", "blacklist"],
        domains: SequenceNotStr[str] | Omit = omit,
        ip_ranges: SequenceNotStr[str] | Omit = omit,
        limit_bw_kbps: Literal[512, 1024] | Omit = omit,
        services: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TrafficPolicyProfileCreateResponse:
        """Create a new traffic policy profile.

        At least one of `services`, `ip_ranges`, or
        `domains` must be provided.

        Args:
          type: The type of the traffic policy profile.

          domains: Array of domain names.

          ip_ranges: Array of IP ranges in CIDR notation.

          limit_bw_kbps: Bandwidth limit in kbps. Must be 512 or 1024.

          services: Array of PCEF service IDs to include in the profile.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/traffic_policy_profiles",
            body=await async_maybe_transform(
                {
                    "type": type,
                    "domains": domains,
                    "ip_ranges": ip_ranges,
                    "limit_bw_kbps": limit_bw_kbps,
                    "services": services,
                },
                traffic_policy_profile_create_params.TrafficPolicyProfileCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TrafficPolicyProfileCreateResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TrafficPolicyProfileRetrieveResponse:
        """
        Returns the details regarding a specific traffic policy profile.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            path_template("/traffic_policy_profiles/{id}", id=id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TrafficPolicyProfileRetrieveResponse,
        )

    async def update(
        self,
        id: str,
        *,
        domains: SequenceNotStr[str] | Omit = omit,
        ip_ranges: SequenceNotStr[str] | Omit = omit,
        limit_bw_kbps: Optional[Literal[512, 1024]] | Omit = omit,
        services: SequenceNotStr[str] | Omit = omit,
        type: Literal["whitelist", "blacklist", "throttling"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TrafficPolicyProfileUpdateResponse:
        """
        Updates a traffic policy profile.

        Args:
          domains: Array of domain names.

          ip_ranges: Array of IP ranges in CIDR notation.

          limit_bw_kbps: Bandwidth limit in kbps. Must be 512 or 1024, or null to remove.

          services: Array of PCEF service IDs to include in the profile.

          type: The type of the traffic policy profile.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._patch(
            path_template("/traffic_policy_profiles/{id}", id=id),
            body=await async_maybe_transform(
                {
                    "domains": domains,
                    "ip_ranges": ip_ranges,
                    "limit_bw_kbps": limit_bw_kbps,
                    "services": services,
                    "type": type,
                },
                traffic_policy_profile_update_params.TrafficPolicyProfileUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TrafficPolicyProfileUpdateResponse,
        )

    def list(
        self,
        *,
        filter_service: str | Omit = omit,
        filter_type: Literal["whitelist", "blacklist", "throttling"] | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        sort: Literal["service", "-service", "type", "-type"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[TrafficPolicyProfile, AsyncDefaultFlatPagination[TrafficPolicyProfile]]:
        """
        Get all traffic policy profiles belonging to the user that match the given
        filters.

        Args:
          filter_service: Filter by service ID.

          filter_type: Filter by traffic policy profile type.

          page_number: The page number to load.

          page_size: The size of the page.

          sort: Sorts traffic policy profiles by the given field. Defaults to ascending order
              unless field is prefixed with a minus sign.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/traffic_policy_profiles",
            page=AsyncDefaultFlatPagination[TrafficPolicyProfile],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter_service": filter_service,
                        "filter_type": filter_type,
                        "page_number": page_number,
                        "page_size": page_size,
                        "sort": sort,
                    },
                    traffic_policy_profile_list_params.TrafficPolicyProfileListParams,
                ),
            ),
            model=TrafficPolicyProfile,
        )

    async def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TrafficPolicyProfileDeleteResponse:
        """
        Deletes the traffic policy profile.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            path_template("/traffic_policy_profiles/{id}", id=id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TrafficPolicyProfileDeleteResponse,
        )

    def list_services(
        self,
        *,
        filter_group: str | Omit = omit,
        filter_name: str | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[
        TrafficPolicyProfileListServicesResponse, AsyncDefaultFlatPagination[TrafficPolicyProfileListServicesResponse]
    ]:
        """
        Get all available PCEF services that can be used in traffic policy profiles.

        Args:
          filter_group: Filter services by group.

          filter_name: Filter services by name.

          page_number: The page number to load.

          page_size: The size of the page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/traffic_policy_profiles/services",
            page=AsyncDefaultFlatPagination[TrafficPolicyProfileListServicesResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter_group": filter_group,
                        "filter_name": filter_name,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    traffic_policy_profile_list_services_params.TrafficPolicyProfileListServicesParams,
                ),
            ),
            model=TrafficPolicyProfileListServicesResponse,
        )


class TrafficPolicyProfilesResourceWithRawResponse:
    def __init__(self, traffic_policy_profiles: TrafficPolicyProfilesResource) -> None:
        self._traffic_policy_profiles = traffic_policy_profiles

        self.create = to_raw_response_wrapper(
            traffic_policy_profiles.create,
        )
        self.retrieve = to_raw_response_wrapper(
            traffic_policy_profiles.retrieve,
        )
        self.update = to_raw_response_wrapper(
            traffic_policy_profiles.update,
        )
        self.list = to_raw_response_wrapper(
            traffic_policy_profiles.list,
        )
        self.delete = to_raw_response_wrapper(
            traffic_policy_profiles.delete,
        )
        self.list_services = to_raw_response_wrapper(
            traffic_policy_profiles.list_services,
        )


class AsyncTrafficPolicyProfilesResourceWithRawResponse:
    def __init__(self, traffic_policy_profiles: AsyncTrafficPolicyProfilesResource) -> None:
        self._traffic_policy_profiles = traffic_policy_profiles

        self.create = async_to_raw_response_wrapper(
            traffic_policy_profiles.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            traffic_policy_profiles.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            traffic_policy_profiles.update,
        )
        self.list = async_to_raw_response_wrapper(
            traffic_policy_profiles.list,
        )
        self.delete = async_to_raw_response_wrapper(
            traffic_policy_profiles.delete,
        )
        self.list_services = async_to_raw_response_wrapper(
            traffic_policy_profiles.list_services,
        )


class TrafficPolicyProfilesResourceWithStreamingResponse:
    def __init__(self, traffic_policy_profiles: TrafficPolicyProfilesResource) -> None:
        self._traffic_policy_profiles = traffic_policy_profiles

        self.create = to_streamed_response_wrapper(
            traffic_policy_profiles.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            traffic_policy_profiles.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            traffic_policy_profiles.update,
        )
        self.list = to_streamed_response_wrapper(
            traffic_policy_profiles.list,
        )
        self.delete = to_streamed_response_wrapper(
            traffic_policy_profiles.delete,
        )
        self.list_services = to_streamed_response_wrapper(
            traffic_policy_profiles.list_services,
        )


class AsyncTrafficPolicyProfilesResourceWithStreamingResponse:
    def __init__(self, traffic_policy_profiles: AsyncTrafficPolicyProfilesResource) -> None:
        self._traffic_policy_profiles = traffic_policy_profiles

        self.create = async_to_streamed_response_wrapper(
            traffic_policy_profiles.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            traffic_policy_profiles.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            traffic_policy_profiles.update,
        )
        self.list = async_to_streamed_response_wrapper(
            traffic_policy_profiles.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            traffic_policy_profiles.delete,
        )
        self.list_services = async_to_streamed_response_wrapper(
            traffic_policy_profiles.list_services,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Optional

from ..._models import BaseModel

__all__ = ["ToolListResponse"]


class ToolListResponse(BaseModel):
    id: str

    tool_definition: Dict[str, object]

    type: str

    created_at: Optional[str] = None

    display_name: Optional[str] = None

    timeout_ms: Optional[int] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ...._models import BaseModel

__all__ = ["PaginationMetaSimple"]


class PaginationMetaSimple(BaseModel):
    page_number: int

    total_pages: int

    page_size: Optional[int] = None

    total_results: Optional[int] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Iterable
from typing_extensions import Literal, Required, TypeAlias, TypedDict

from .._types import SequenceNotStr

__all__ = [
    "WhatsappMessageTemplateUpdateParams",
    "Component",
    "ComponentHeader",
    "ComponentHeaderExample",
    "ComponentBody",
    "ComponentBodyExample",
    "ComponentFooter",
    "ComponentButtons",
    "ComponentButtonsButton",
    "ComponentCarousel",
    "ComponentCarouselCard",
]


class WhatsappMessageTemplateUpdateParams(TypedDict, total=False):
    category: Literal["MARKETING", "UTILITY", "AUTHENTICATION"]

    components: Iterable[Component]
    """Updated template components. Same structure as the create request."""


class ComponentHeaderExample(TypedDict, total=False):
    """Sample values for header variables."""

    header_handle: SequenceNotStr[str]
    """Media handle for IMAGE, VIDEO, or DOCUMENT headers."""

    header_text: SequenceNotStr[str]
    """Sample values for text header variables."""


class ComponentHeader(TypedDict, total=False):
    """Optional header displayed at the top of the message."""

    format: Required[Literal["TEXT", "IMAGE", "VIDEO", "DOCUMENT", "LOCATION"]]
    """
    Header format type: TEXT (supports one variable), IMAGE, VIDEO, DOCUMENT, or
    LOCATION.
    """

    type: Required[Literal["HEADER"]]

    example: ComponentHeaderExample
    """Sample values for header variables."""

    text: str
    """Header text.

    Required when format is TEXT. Supports one variable ({{1}}). Variables cannot be
    at the start or end.
    """


class ComponentBodyExample(TypedDict, total=False):
    """Sample values for body variables. Required when body text contains parameters."""

    body_text: Iterable[SequenceNotStr[str]]
    """Array containing one array of sample values, one per variable in order."""


class ComponentBody(TypedDict, total=False):
    """The main text content of the message.

    Supports multiple variable parameters ({{1}}, {{2}}, etc.). Variables cannot be at the start or end. Maximum 1024 characters.
    """

    type: Required[Literal["BODY"]]

    example: ComponentBodyExample
    """Sample values for body variables. Required when body text contains parameters."""

    text: str
    """Body text content.

    Use {{1}}, {{2}}, etc. for variable placeholders. Required for MARKETING and
    UTILITY templates. Optional for AUTHENTICATION templates where Meta provides the
    built-in OTP body.
    """


class ComponentFooter(TypedDict, total=False):
    """Optional footer displayed at the bottom of the message.

    Does not support variables.
    """

    type: Required[Literal["FOOTER"]]

    code_expiration_minutes: int
    """OTP code expiration time in minutes.

    Used in AUTHENTICATION template footers instead of free-form text.
    """

    text: str
    """Footer text. Maximum 60 characters. For non-authentication templates."""


class ComponentButtonsButton(TypedDict, total=False):
    type: Required[Literal["URL", "PHONE_NUMBER", "QUICK_REPLY", "OTP", "COPY_CODE", "FLOW"]]

    autofill_text: str
    """Custom autofill button text for ONE_TAP OTP buttons."""

    example: SequenceNotStr[str]
    """Sample values for URL variable."""

    flow_action: Literal["navigate", "data_exchange"]
    """Flow action type for FLOW-type buttons."""

    flow_id: str
    """Flow ID for FLOW-type buttons."""

    navigate_screen: str
    """Target screen name for FLOW buttons with navigate action."""

    otp_type: Literal["COPY_CODE", "ONE_TAP"]

    package_name: str
    """Android package name. Required for ONE_TAP OTP buttons."""

    phone_number: str
    """Phone number in E.164 format."""

    signature_hash: str
    """Android app signing key hash. Required for ONE_TAP OTP buttons."""

    text: str
    """Button label text.

    Maximum 25 characters. Required for URL, PHONE_NUMBER, and QUICK_REPLY buttons.
    Not required for OTP buttons (Meta supplies the label).
    """

    url: str
    """URL for URL-type buttons. Supports one variable ({{1}})."""

    zero_tap_terms_accepted: bool
    """Whether zero-tap terms have been accepted."""


class ComponentButtons(TypedDict, total=False):
    """Optional interactive buttons. Maximum 3 buttons per template."""

    buttons: Required[Iterable[ComponentButtonsButton]]
    """Array of button objects. Meta supports various combinations of button types."""

    type: Required[Literal["BUTTONS"]]


class ComponentCarouselCard(TypedDict, total=False):
    components: Iterable[Dict[str, object]]


class ComponentCarousel(TypedDict, total=False):
    """Carousel component for multi-card templates.

    Each card can contain its own header, body, and buttons.
    """

    cards: Required[Iterable[ComponentCarouselCard]]
    """Array of card objects, each with its own components."""

    type: Required[Literal["CAROUSEL"]]


Component: TypeAlias = Union[ComponentHeader, ComponentBody, ComponentFooter, ComponentButtons, ComponentCarousel]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .credit_account_settle_params import CreditAccountSettleParams as CreditAccountSettleParams
from .credit_account_settle_response import CreditAccountSettleResponse as CreditAccountSettleResponse
from .credit_account_create_quote_params import CreditAccountCreateQuoteParams as CreditAccountCreateQuoteParams
from .credit_account_create_quote_response import CreditAccountCreateQuoteResponse as CreditAccountCreateQuoteResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["CallCostWebhookEvent", "Data", "DataPayload", "DataPayloadCostPart"]


class DataPayloadCostPart(BaseModel):
    billed_duration_secs: Optional[int] = None
    """The billed duration in seconds for this part of the call."""

    call_part: Optional[str] = None
    """The product component this cost applies to.

    Values are determined by the billing system (e.g. sip-trunking, call-control,
    call-recording). Not a fixed set — new values may appear as products evolve.
    """

    cost: Optional[str] = None
    """The cost for this part of the call."""

    currency: Optional[str] = None
    """The currency of the cost."""

    rate: Optional[str] = None
    """The per-minute rate applied."""


class DataPayload(BaseModel):
    billed_duration_secs: Optional[int] = None
    """The longest billed duration across all cost parts, in seconds."""

    billing_group_id: Optional[str] = None
    """Identifies the billing group associated with the call."""

    call_control_id: Optional[str] = None
    """Call ID used to issue commands via Call Control API."""

    call_leg_id: Optional[str] = None
    """ID that is unique to the call and can be used to correlate webhook events."""

    call_session_id: Optional[str] = None
    """
    ID that is unique to the call session and can be used to correlate webhook
    events. Call session is a group of related call legs that logically belong to
    the same phone call, e.g. an inbound and outbound leg of a transferred call.
    """

    client_state: Optional[str] = None
    """State received from a command. Base64-encoded."""

    connection_id: Optional[str] = None
    """Call Control App ID (formerly Telnyx connection ID) used in the call."""

    cost_parts: Optional[List[DataPayloadCostPart]] = None
    """Breakdown of costs by call part."""

    occurred_at: Optional[datetime] = None
    """ISO 8601 datetime of when the event occurred."""

    status: Optional[Literal["success", "error"]] = None
    """The status of the cost calculation (`success` or `error`)."""

    total_cost: Optional[str] = None
    """The total cost of the call."""


class Data(BaseModel):
    id: Optional[str] = None
    """Unique identifier of the event."""

    event_type: Optional[Literal["call.cost"]] = None
    """The type of event being delivered."""

    occurred_at: Optional[datetime] = None
    """ISO 8601 datetime of when the event occurred."""

    payload: Optional[DataPayload] = None

    record_type: Optional[Literal["event"]] = None
    """Identifies the type of the resource."""


class CallCostWebhookEvent(BaseModel):
    data: Optional[Data] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .x402 import (
    X402Resource,
    AsyncX402Resource,
    X402ResourceWithRawResponse,
    AsyncX402ResourceWithRawResponse,
    X402ResourceWithStreamingResponse,
    AsyncX402ResourceWithStreamingResponse,
)
from .credit_account import (
    CreditAccountResource,
    AsyncCreditAccountResource,
    CreditAccountResourceWithRawResponse,
    AsyncCreditAccountResourceWithRawResponse,
    CreditAccountResourceWithStreamingResponse,
    AsyncCreditAccountResourceWithStreamingResponse,
)

__all__ = [
    "CreditAccountResource",
    "AsyncCreditAccountResource",
    "CreditAccountResourceWithRawResponse",
    "AsyncCreditAccountResourceWithRawResponse",
    "CreditAccountResourceWithStreamingResponse",
    "AsyncCreditAccountResourceWithStreamingResponse",
    "X402Resource",
    "AsyncX402Resource",
    "X402ResourceWithRawResponse",
    "AsyncX402ResourceWithRawResponse",
    "X402ResourceWithStreamingResponse",
    "AsyncX402ResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from telnyx.types.whatsapp import (
    PhoneNumberListResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestPhoneNumbers:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list(self, client: Telnyx) -> None:
        phone_number = client.whatsapp.phone_numbers.list()
        assert_matches_type(SyncDefaultFlatPagination[PhoneNumberListResponse], phone_number, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Telnyx) -> None:
        phone_number = client.whatsapp.phone_numbers.list(
            page_number=0,
            page_size=0,
        )
        assert_matches_type(SyncDefaultFlatPagination[PhoneNumberListResponse], phone_number, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Telnyx) -> None:
        response = client.whatsapp.phone_numbers.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        phone_number = response.parse()
        assert_matches_type(SyncDefaultFlatPagination[PhoneNumberListResponse], phone_number, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Telnyx) -> None:
        with client.whatsapp.phone_numbers.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            phone_number = response.parse()
            assert_matches_type(SyncDefaultFlatPagination[PhoneNumberListResponse], phone_number, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_delete(self, client: Telnyx) -> None:
        phone_number = client.whatsapp.phone_numbers.delete(
            "phone_number",
        )
        assert phone_number is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Telnyx) -> None:
        response = client.whatsapp.phone_numbers.with_raw_response.delete(
            "phone_number",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        phone_number = response.parse()
        assert phone_number is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Telnyx) -> None:
        with client.whatsapp.phone_numbers.with_streaming_response.delete(
            "phone_number",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            phone_number = response.parse()
            assert phone_number is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `phone_number` but received ''"):
            client.whatsapp.phone_numbers.with_raw_response.delete(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_resend_verification(self, client: Telnyx) -> None:
        phone_number = client.whatsapp.phone_numbers.resend_verification(
            phone_number="phone_number",
        )
        assert phone_number is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_resend_verification_with_all_params(self, client: Telnyx) -> None:
        phone_number = client.whatsapp.phone_numbers.resend_verification(
            phone_number="phone_number",
            verification_method="sms",
        )
        assert phone_number is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_resend_verification(self, client: Telnyx) -> None:
        response = client.whatsapp.phone_numbers.with_raw_response.resend_verification(
            phone_number="phone_number",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        phone_number = response.parse()
        assert phone_number is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_resend_verification(self, client: Telnyx) -> None:
        with client.whatsapp.phone_numbers.with_streaming_response.resend_verification(
            phone_number="phone_number",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            phone_number = response.parse()
            assert phone_number is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_resend_verification(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `phone_number` but received ''"):
            client.whatsapp.phone_numbers.with_raw_response.resend_verification(
                phone_number="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_verify(self, client: Telnyx) -> None:
        phone_number = client.whatsapp.phone_numbers.verify(
            phone_number="phone_number",
            code="code",
        )
        assert phone_number is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_verify(self, client: Telnyx) -> None:
        response = client.whatsapp.phone_numbers.with_raw_response.verify(
            phone_number="phone_number",
            code="code",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        phone_number = response.parse()
        assert phone_number is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_verify(self, client: Telnyx) -> None:
        with client.whatsapp.phone_numbers.with_streaming_response.verify(
            phone_number="phone_number",
            code="code",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            phone_number = response.parse()
            assert phone_number is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_verify(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `phone_number` but received ''"):
            client.whatsapp.phone_numbers.with_raw_response.verify(
                phone_number="",
                code="code",
            )


class TestAsyncPhoneNumbers:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncTelnyx) -> None:
        phone_number = await async_client.whatsapp.phone_numbers.list()
        assert_matches_type(AsyncDefaultFlatPagination[PhoneNumberListResponse], phone_number, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncTelnyx) -> None:
        phone_number = await async_client.whatsapp.phone_numbers.list(
            page_number=0,
            page_size=0,
        )
        assert_matches_type(AsyncDefaultFlatPagination[PhoneNumberListResponse], phone_number, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.whatsapp.phone_numbers.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        phone_number = await response.parse()
        assert_matches_type(AsyncDefaultFlatPagination[PhoneNumberListResponse], phone_number, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTelnyx) -> None:
        async with async_client.whatsapp.phone_numbers.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            phone_number = await response.parse()
            assert_matches_type(AsyncDefaultFlatPagination[PhoneNumberListResponse], phone_number, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncTelnyx) -> None:
        phone_number = await async_client.whatsapp.phone_numbers.delete(
            "phone_number",
        )
        assert phone_number is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.whatsapp.phone_numbers.with_raw_response.delete(
            "phone_number",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        phone_number = await response.parse()
        assert phone_number is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncTelnyx) -> None:
        async with async_client.whatsapp.phone_numbers.with_streaming_response.delete(
            "phone_number",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            phone_number = await response.parse()
            assert phone_number is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `phone_number` but received ''"):
            await async_client.whatsapp.phone_numbers.with_raw_response.delete(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_resend_verification(self, async_client: AsyncTelnyx) -> None:
        phone_number = await async_client.whatsapp.phone_numbers.resend_verification(
            phone_number="phone_number",
        )
        assert phone_number is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_resend_verification_with_all_params(self, async_client: AsyncTelnyx) -> None:
        phone_number = await async_client.whatsapp.phone_numbers.resend_verification(
            phone_number="phone_number",
            verification_method="sms",
        )
        assert phone_number is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_resend_verification(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.whatsapp.phone_numbers.with_raw_response.resend_verification(
            phone_number="phone_number",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        phone_number = await response.parse()
        assert phone_number is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_resend_verification(self, async_client: AsyncTelnyx) -> None:
        async with async_client.whatsapp.phone_numbers.with_streaming_response.resend_verification(
            phone_number="phone_number",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            phone_number = await response.parse()
            assert phone_number is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_resend_verification(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `phone_number` but received ''"):
            await async_client.whatsapp.phone_numbers.with_raw_response.resend_verification(
                phone_number="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_verify(self, async_client: AsyncTelnyx) -> None:
        phone_number = await async_client.whatsapp.phone_numbers.verify(
            phone_number="phone_number",
            code="code",
        )
        assert phone_number is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_verify(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.whatsapp.phone_numbers.with_raw_response.verify(
            phone_number="phone_number",
            code="code",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        phone_number = await response.parse()
        assert phone_number is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_verify(self, async_client: AsyncTelnyx) -> None:
        async with async_client.whatsapp.phone_numbers.with_streaming_response.verify(
            phone_number="phone_number",
            code="code",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            phone_number = await response.parse()
            assert phone_number is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_verify(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `phone_number` but received ''"):
            await async_client.whatsapp.phone_numbers.with_raw_response.verify(
                phone_number="",
                code="code",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["TrafficPolicyProfile"]


class TrafficPolicyProfile(BaseModel):
    id: Optional[str] = None
    """Identifies the resource."""

    created_at: Optional[str] = None
    """ISO 8601 formatted date-time indicating when the resource was created."""

    domains: Optional[List[str]] = None
    """Array of domain names."""

    ip_ranges: Optional[List[str]] = None
    """Array of IP ranges in CIDR notation."""

    limit_bw_kbps: Optional[int] = None
    """Bandwidth limit in kbps."""

    record_type: Optional[str] = None

    services: Optional[List[str]] = None
    """Array of PCEF service IDs included in the profile."""

    type: Optional[Literal["whitelist", "blacklist", "throttling"]] = None
    """The type of the traffic policy profile."""

    updated_at: Optional[str] = None
    """ISO 8601 formatted date-time indicating when the resource was updated."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict
from typing_extensions import TypedDict

__all__ = ["ToolUpdateParams"]


class ToolUpdateParams(TypedDict, total=False):
    display_name: str

    function: Dict[str, object]

    handoff: Dict[str, object]

    invite: Dict[str, object]

    retrieval: Dict[str, object]

    timeout_ms: int

    type: str

    webhook: Dict[str, object]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["CreditAccountSettleParams"]


class CreditAccountSettleParams(TypedDict, total=False):
    id: Required[str]
    """The quote ID to settle."""

    payment_signature: str
    """Base64-encoded signed payment authorization (x402 PaymentPayload).

    Can alternatively be provided via the PAYMENT-SIGNATURE header.
    """

    header_payment_signature: Annotated[str, PropertyInfo(alias="PAYMENT-SIGNATURE")]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

from .._types import SequenceNotStr

__all__ = ["TrafficPolicyProfileCreateParams"]


class TrafficPolicyProfileCreateParams(TypedDict, total=False):
    type: Required[Literal["whitelist", "blacklist"]]
    """The type of the traffic policy profile."""

    domains: SequenceNotStr[str]
    """Array of domain names."""

    ip_ranges: SequenceNotStr[str]
    """Array of IP ranges in CIDR notation."""

    limit_bw_kbps: Literal[512, 1024]
    """Bandwidth limit in kbps. Must be 512 or 1024."""

    services: SequenceNotStr[str]
    """Array of PCEF service IDs to include in the profile."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Iterable
from typing_extensions import Literal, Required, TypeAlias, TypedDict

from ..._types import SequenceNotStr

__all__ = [
    "TemplateCreateParams",
    "Component",
    "ComponentHeader",
    "ComponentHeaderExample",
    "ComponentBody",
    "ComponentBodyExample",
    "ComponentFooter",
    "ComponentButtons",
    "ComponentButtonsButton",
    "ComponentCarousel",
    "ComponentCarouselCard",
]


class TemplateCreateParams(TypedDict, total=False):
    category: Required[Literal["MARKETING", "UTILITY", "AUTHENTICATION"]]
    """Template category: AUTHENTICATION, UTILITY, or MARKETING."""

    components: Required[Iterable[Component]]
    """Template components defining message structure.

    Passed through to Meta Graph API. Templates with variables must include example
    values. Supports HEADER, BODY, FOOTER, BUTTONS, CAROUSEL and any future Meta
    component types.
    """

    language: Required[str]
    """Template language code (e.g. en_US, es, pt_BR)."""

    name: Required[str]
    """Template name. Lowercase letters, numbers, and underscores only."""

    waba_id: Required[str]
    """The WhatsApp Business Account ID."""


class ComponentHeaderExample(TypedDict, total=False):
    """Sample values for header variables."""

    header_handle: SequenceNotStr[str]
    """Media handle for IMAGE, VIDEO, or DOCUMENT headers."""

    header_text: SequenceNotStr[str]
    """Sample values for text header variables."""


class ComponentHeader(TypedDict, total=False):
    """Optional header displayed at the top of the message."""

    format: Required[Literal["TEXT", "IMAGE", "VIDEO", "DOCUMENT", "LOCATION"]]
    """
    Header format type: TEXT (supports one variable), IMAGE, VIDEO, DOCUMENT, or
    LOCATION.
    """

    type: Required[Literal["HEADER"]]

    example: ComponentHeaderExample
    """Sample values for header variables."""

    text: str
    """Header text.

    Required when format is TEXT. Supports one variable ({{1}}). Variables cannot be
    at the start or end.
    """


class ComponentBodyExample(TypedDict, total=False):
    """Sample values for body variables. Required when body text contains parameters."""

    body_text: Iterable[SequenceNotStr[str]]
    """Array containing one array of sample values, one per variable in order."""


class ComponentBody(TypedDict, total=False):
    """The main text content of the message.

    Supports multiple variable parameters ({{1}}, {{2}}, etc.). Variables cannot be at the start or end. Maximum 1024 characters.
    """

    type: Required[Literal["BODY"]]

    example: ComponentBodyExample
    """Sample values for body variables. Required when body text contains parameters."""

    text: str
    """Body text content.

    Use {{1}}, {{2}}, etc. for variable placeholders. Required for MARKETING and
    UTILITY templates. Optional for AUTHENTICATION templates where Meta provides the
    built-in OTP body.
    """


class ComponentFooter(TypedDict, total=False):
    """Optional footer displayed at the bottom of the message.

    Does not support variables.
    """

    type: Required[Literal["FOOTER"]]

    code_expiration_minutes: int
    """OTP code expiration time in minutes.

    Used in AUTHENTICATION template footers instead of free-form text.
    """

    text: str
    """Footer text. Maximum 60 characters. For non-authentication templates."""


class ComponentButtonsButton(TypedDict, total=False):
    type: Required[Literal["URL", "PHONE_NUMBER", "QUICK_REPLY", "OTP", "COPY_CODE", "FLOW"]]

    autofill_text: str
    """Custom autofill button text for ONE_TAP OTP buttons."""

    example: SequenceNotStr[str]
    """Sample values for URL variable."""

    flow_action: Literal["navigate", "data_exchange"]
    """Flow action type for FLOW-type buttons."""

    flow_id: str
    """Flow ID for FLOW-type buttons."""

    navigate_screen: str
    """Target screen name for FLOW buttons with navigate action."""

    otp_type: Literal["COPY_CODE", "ONE_TAP"]

    package_name: str
    """Android package name. Required for ONE_TAP OTP buttons."""

    phone_number: str
    """Phone number in E.164 format."""

    signature_hash: str
    """Android app signing key hash. Required for ONE_TAP OTP buttons."""

    text: str
    """Button label text.

    Maximum 25 characters. Required for URL, PHONE_NUMBER, and QUICK_REPLY buttons.
    Not required for OTP buttons (Meta supplies the label).
    """

    url: str
    """URL for URL-type buttons. Supports one variable ({{1}})."""

    zero_tap_terms_accepted: bool
    """Whether zero-tap terms have been accepted."""


class ComponentButtons(TypedDict, total=False):
    """Optional interactive buttons. Maximum 3 buttons per template."""

    buttons: Required[Iterable[ComponentButtonsButton]]
    """Array of button objects. Meta supports various combinations of button types."""

    type: Required[Literal["BUTTONS"]]


class ComponentCarouselCard(TypedDict, total=False):
    components: Iterable[Dict[str, object]]


class ComponentCarousel(TypedDict, total=False):
    """Carousel component for multi-card templates.

    Each card can contain its own header, body, and buttons.
    """

    cards: Required[Iterable[ComponentCarouselCard]]
    """Array of card objects, each with its own components."""

    type: Required[Literal["CAROUSEL"]]


Component: TypeAlias = Union[ComponentHeader, ComponentBody, ComponentFooter, ComponentButtons, ComponentCarousel]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .traffic_policy_profile import TrafficPolicyProfile

__all__ = ["TrafficPolicyProfileCreateResponse"]


class TrafficPolicyProfileCreateResponse(BaseModel):
    data: Optional[TrafficPolicyProfile] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

from ..._types import SequenceNotStr

__all__ = ["ActionDeleteParams"]


class ActionDeleteParams(TypedDict, total=False):
    ids: Required[SequenceNotStr[str]]
    """List of call recording IDs to delete."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform, strip_not_given, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...types.x402 import credit_account_settle_params, credit_account_create_quote_params
from ..._base_client import make_request_options
from ...types.x402.credit_account_settle_response import CreditAccountSettleResponse
from ...types.x402.credit_account_create_quote_response import CreditAccountCreateQuoteResponse

__all__ = ["CreditAccountResource", "AsyncCreditAccountResource"]


class CreditAccountResource(SyncAPIResource):
    """Operations for x402 cryptocurrency payment transactions.

    Fund your Telnyx account using USDC stablecoin payments via the x402 protocol.
    """

    @cached_property
    def with_raw_response(self) -> CreditAccountResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return CreditAccountResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CreditAccountResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return CreditAccountResourceWithStreamingResponse(self)

    def create_quote(
        self,
        *,
        amount_usd: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CreditAccountCreateQuoteResponse:
        """Creates a payment quote for the specified USD amount.

        Returns payment details
        including the x402 payment requirements, network, and expiration time. The quote
        must be settled before it expires.

        Args:
          amount_usd: Amount in USD to fund (minimum 5.00, maximum 10000.00).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v2/x402/credit_account/quote",
            body=maybe_transform(
                {"amount_usd": amount_usd}, credit_account_create_quote_params.CreditAccountCreateQuoteParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CreditAccountCreateQuoteResponse,
        )

    def settle(
        self,
        *,
        id: str,
        payment_signature: str | Omit = omit,
        header_payment_signature: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CreditAccountSettleResponse:
        """
        Settles an x402 payment using the quote ID and a signed payment authorization.
        The payment signature can be provided via the `PAYMENT-SIGNATURE` header or the
        `payment_signature` body parameter. Settlement is idempotent — submitting the
        same quote ID multiple times returns the existing transaction.

        Args:
          id: The quote ID to settle.

          payment_signature: Base64-encoded signed payment authorization (x402 PaymentPayload). Can
              alternatively be provided via the PAYMENT-SIGNATURE header.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {**strip_not_given({"PAYMENT-SIGNATURE": header_payment_signature}), **(extra_headers or {})}
        return self._post(
            "/v2/x402/credit_account",
            body=maybe_transform(
                {
                    "id": id,
                    "payment_signature": payment_signature,
                },
                credit_account_settle_params.CreditAccountSettleParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CreditAccountSettleResponse,
        )


class AsyncCreditAccountResource(AsyncAPIResource):
    """Operations for x402 cryptocurrency payment transactions.

    Fund your Telnyx account using USDC stablecoin payments via the x402 protocol.
    """

    @cached_property
    def with_raw_response(self) -> AsyncCreditAccountResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncCreditAccountResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCreditAccountResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncCreditAccountResourceWithStreamingResponse(self)

    async def create_quote(
        self,
        *,
        amount_usd: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CreditAccountCreateQuoteResponse:
        """Creates a payment quote for the specified USD amount.

        Returns payment details
        including the x402 payment requirements, network, and expiration time. The quote
        must be settled before it expires.

        Args:
          amount_usd: Amount in USD to fund (minimum 5.00, maximum 10000.00).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v2/x402/credit_account/quote",
            body=await async_maybe_transform(
                {"amount_usd": amount_usd}, credit_account_create_quote_params.CreditAccountCreateQuoteParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CreditAccountCreateQuoteResponse,
        )

    async def settle(
        self,
        *,
        id: str,
        payment_signature: str | Omit = omit,
        header_payment_signature: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CreditAccountSettleResponse:
        """
        Settles an x402 payment using the quote ID and a signed payment authorization.
        The payment signature can be provided via the `PAYMENT-SIGNATURE` header or the
        `payment_signature` body parameter. Settlement is idempotent — submitting the
        same quote ID multiple times returns the existing transaction.

        Args:
          id: The quote ID to settle.

          payment_signature: Base64-encoded signed payment authorization (x402 PaymentPayload). Can
              alternatively be provided via the PAYMENT-SIGNATURE header.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {**strip_not_given({"PAYMENT-SIGNATURE": header_payment_signature}), **(extra_headers or {})}
        return await self._post(
            "/v2/x402/credit_account",
            body=await async_maybe_transform(
                {
                    "id": id,
                    "payment_signature": payment_signature,
                },
                credit_account_settle_params.CreditAccountSettleParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CreditAccountSettleResponse,
        )


class CreditAccountResourceWithRawResponse:
    def __init__(self, credit_account: CreditAccountResource) -> None:
        self._credit_account = credit_account

        self.create_quote = to_raw_response_wrapper(
            credit_account.create_quote,
        )
        self.settle = to_raw_response_wrapper(
            credit_account.settle,
        )


class AsyncCreditAccountResourceWithRawResponse:
    def __init__(self, credit_account: AsyncCreditAccountResource) -> None:
        self._credit_account = credit_account

        self.create_quote = async_to_raw_response_wrapper(
            credit_account.create_quote,
        )
        self.settle = async_to_raw_response_wrapper(
            credit_account.settle,
        )


class CreditAccountResourceWithStreamingResponse:
    def __init__(self, credit_account: CreditAccountResource) -> None:
        self._credit_account = credit_account

        self.create_quote = to_streamed_response_wrapper(
            credit_account.create_quote,
        )
        self.settle = to_streamed_response_wrapper(
            credit_account.settle,
        )


class AsyncCreditAccountResourceWithStreamingResponse:
    def __init__(self, credit_account: AsyncCreditAccountResource) -> None:
        self._credit_account = credit_account

        self.create_quote = async_to_streamed_response_wrapper(
            credit_account.create_quote,
        )
        self.settle = async_to_streamed_response_wrapper(
            credit_account.settle,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["ProfileAssignmentPhoneNumbers"]


class ProfileAssignmentPhoneNumbers(BaseModel):
    phone_number: str = FieldInfo(alias="phoneNumber")
    """The phone number that the status is being checked for."""

    status: str
    """The status of the associated phone number assignment."""

    task_id: str = FieldInfo(alias="taskId")
    """The ID of the task associated with the phone number."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.types import (
    TrafficPolicyProfile,
    TrafficPolicyProfileCreateResponse,
    TrafficPolicyProfileDeleteResponse,
    TrafficPolicyProfileUpdateResponse,
    TrafficPolicyProfileRetrieveResponse,
    TrafficPolicyProfileListServicesResponse,
)
from telnyx.pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestTrafficPolicyProfiles:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create(self, client: Telnyx) -> None:
        traffic_policy_profile = client.traffic_policy_profiles.create(
            type="whitelist",
        )
        assert_matches_type(TrafficPolicyProfileCreateResponse, traffic_policy_profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Telnyx) -> None:
        traffic_policy_profile = client.traffic_policy_profiles.create(
            type="whitelist",
            domains=["www.hbomax.com", "netflix.com"],
            ip_ranges=["10.64.0.0/24", "10.64.0.0/25"],
            limit_bw_kbps=512,
            services=["service_123", "service_456"],
        )
        assert_matches_type(TrafficPolicyProfileCreateResponse, traffic_policy_profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Telnyx) -> None:
        response = client.traffic_policy_profiles.with_raw_response.create(
            type="whitelist",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        traffic_policy_profile = response.parse()
        assert_matches_type(TrafficPolicyProfileCreateResponse, traffic_policy_profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Telnyx) -> None:
        with client.traffic_policy_profiles.with_streaming_response.create(
            type="whitelist",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            traffic_policy_profile = response.parse()
            assert_matches_type(TrafficPolicyProfileCreateResponse, traffic_policy_profile, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Telnyx) -> None:
        traffic_policy_profile = client.traffic_policy_profiles.retrieve(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )
        assert_matches_type(TrafficPolicyProfileRetrieveResponse, traffic_policy_profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Telnyx) -> None:
        response = client.traffic_policy_profiles.with_raw_response.retrieve(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        traffic_policy_profile = response.parse()
        assert_matches_type(TrafficPolicyProfileRetrieveResponse, traffic_policy_profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Telnyx) -> None:
        with client.traffic_policy_profiles.with_streaming_response.retrieve(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            traffic_policy_profile = response.parse()
            assert_matches_type(TrafficPolicyProfileRetrieveResponse, traffic_policy_profile, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.traffic_policy_profiles.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_update(self, client: Telnyx) -> None:
        traffic_policy_profile = client.traffic_policy_profiles.update(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )
        assert_matches_type(TrafficPolicyProfileUpdateResponse, traffic_policy_profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_update_with_all_params(self, client: Telnyx) -> None:
        traffic_policy_profile = client.traffic_policy_profiles.update(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            domains=["netflix.com"],
            ip_ranges=["10.64.0.0/24"],
            limit_bw_kbps=1024,
            services=["service_123"],
            type="whitelist",
        )
        assert_matches_type(TrafficPolicyProfileUpdateResponse, traffic_policy_profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: Telnyx) -> None:
        response = client.traffic_policy_profiles.with_raw_response.update(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        traffic_policy_profile = response.parse()
        assert_matches_type(TrafficPolicyProfileUpdateResponse, traffic_policy_profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: Telnyx) -> None:
        with client.traffic_policy_profiles.with_streaming_response.update(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            traffic_policy_profile = response.parse()
            assert_matches_type(TrafficPolicyProfileUpdateResponse, traffic_policy_profile, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_update(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.traffic_policy_profiles.with_raw_response.update(
                id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list(self, client: Telnyx) -> None:
        traffic_policy_profile = client.traffic_policy_profiles.list()
        assert_matches_type(SyncDefaultFlatPagination[TrafficPolicyProfile], traffic_policy_profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Telnyx) -> None:
        traffic_policy_profile = client.traffic_policy_profiles.list(
            filter_service="filter[service]",
            filter_type="whitelist",
            page_number=1,
            page_size=1,
            sort="service",
        )
        assert_matches_type(SyncDefaultFlatPagination[TrafficPolicyProfile], traffic_policy_profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Telnyx) -> None:
        response = client.traffic_policy_profiles.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        traffic_policy_profile = response.parse()
        assert_matches_type(SyncDefaultFlatPagination[TrafficPolicyProfile], traffic_policy_profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Telnyx) -> None:
        with client.traffic_policy_profiles.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            traffic_policy_profile = response.parse()
            assert_matches_type(
                SyncDefaultFlatPagination[TrafficPolicyProfile], traffic_policy_profile, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_delete(self, client: Telnyx) -> None:
        traffic_policy_profile = client.traffic_policy_profiles.delete(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )
        assert_matches_type(TrafficPolicyProfileDeleteResponse, traffic_policy_profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Telnyx) -> None:
        response = client.traffic_policy_profiles.with_raw_response.delete(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        traffic_policy_profile = response.parse()
        assert_matches_type(TrafficPolicyProfileDeleteResponse, traffic_policy_profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Telnyx) -> None:
        with client.traffic_policy_profiles.with_streaming_response.delete(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            traffic_policy_profile = response.parse()
            assert_matches_type(TrafficPolicyProfileDeleteResponse, traffic_policy_profile, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.traffic_policy_profiles.with_raw_response.delete(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_services(self, client: Telnyx) -> None:
        traffic_policy_profile = client.traffic_policy_profiles.list_services()
        assert_matches_type(
            SyncDefaultFlatPagination[TrafficPolicyProfileListServicesResponse],
            traffic_policy_profile,
            path=["response"],
        )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_services_with_all_params(self, client: Telnyx) -> None:
        traffic_policy_profile = client.traffic_policy_profiles.list_services(
            filter_group="filter[group]",
            filter_name="filter[name]",
            page_number=1,
            page_size=1,
        )
        assert_matches_type(
            SyncDefaultFlatPagination[TrafficPolicyProfileListServicesResponse],
            traffic_policy_profile,
            path=["response"],
        )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_list_services(self, client: Telnyx) -> None:
        response = client.traffic_policy_profiles.with_raw_response.list_services()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        traffic_policy_profile = response.parse()
        assert_matches_type(
            SyncDefaultFlatPagination[TrafficPolicyProfileListServicesResponse],
            traffic_policy_profile,
            path=["response"],
        )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_list_services(self, client: Telnyx) -> None:
        with client.traffic_policy_profiles.with_streaming_response.list_services() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            traffic_policy_profile = response.parse()
            assert_matches_type(
                SyncDefaultFlatPagination[TrafficPolicyProfileListServicesResponse],
                traffic_policy_profile,
                path=["response"],
            )

        assert cast(Any, response.is_closed) is True


class TestAsyncTrafficPolicyProfiles:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncTelnyx) -> None:
        traffic_policy_profile = await async_client.traffic_policy_profiles.create(
            type="whitelist",
        )
        assert_matches_type(TrafficPolicyProfileCreateResponse, traffic_policy_profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncTelnyx) -> None:
        traffic_policy_profile = await async_client.traffic_policy_profiles.create(
            type="whitelist",
            domains=["www.hbomax.com", "netflix.com"],
            ip_ranges=["10.64.0.0/24", "10.64.0.0/25"],
            limit_bw_kbps=512,
            services=["service_123", "service_456"],
        )
        assert_matches_type(TrafficPolicyProfileCreateResponse, traffic_policy_profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.traffic_policy_profiles.with_raw_response.create(
            type="whitelist",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        traffic_policy_profile = await response.parse()
        assert_matches_type(TrafficPolicyProfileCreateResponse, traffic_policy_profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncTelnyx) -> None:
        async with async_client.traffic_policy_profiles.with_streaming_response.create(
            type="whitelist",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            traffic_policy_profile = await response.parse()
            assert_matches_type(TrafficPolicyProfileCreateResponse, traffic_policy_profile, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncTelnyx) -> None:
        traffic_policy_profile = await async_client.traffic_policy_profiles.retrieve(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )
        assert_matches_type(TrafficPolicyProfileRetrieveResponse, traffic_policy_profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.traffic_policy_profiles.with_raw_response.retrieve(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        traffic_policy_profile = await response.parse()
        assert_matches_type(TrafficPolicyProfileRetrieveResponse, traffic_policy_profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        async with async_client.traffic_policy_profiles.with_streaming_response.retrieve(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            traffic_policy_profile = await response.parse()
            assert_matches_type(TrafficPolicyProfileRetrieveResponse, traffic_policy_profile, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.traffic_policy_profiles.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncTelnyx) -> None:
        traffic_policy_profile = await async_client.traffic_policy_profiles.update(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )
        assert_matches_type(TrafficPolicyProfileUpdateResponse, traffic_policy_profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncTelnyx) -> None:
        traffic_policy_profile = await async_client.traffic_policy_profiles.update(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            domains=["netflix.com"],
            ip_ranges=["10.64.0.0/24"],
            limit_bw_kbps=1024,
            services=["service_123"],
            type="whitelist",
        )
        assert_matches_type(TrafficPolicyProfileUpdateResponse, traffic_policy_profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.traffic_policy_profiles.with_raw_response.update(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        traffic_policy_profile = await response.parse()
        assert_matches_type(TrafficPolicyProfileUpdateResponse, traffic_policy_profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncTelnyx) -> None:
        async with async_client.traffic_policy_profiles.with_streaming_response.update(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            traffic_policy_profile = await response.parse()
            assert_matches_type(TrafficPolicyProfileUpdateResponse, traffic_policy_profile, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_update(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.traffic_policy_profiles.with_raw_response.update(
                id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncTelnyx) -> None:
        traffic_policy_profile = await async_client.traffic_policy_profiles.list()
        assert_matches_type(AsyncDefaultFlatPagination[TrafficPolicyProfile], traffic_policy_profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncTelnyx) -> None:
        traffic_policy_profile = await async_client.traffic_policy_profiles.list(
            filter_service="filter[service]",
            filter_type="whitelist",
            page_number=1,
            page_size=1,
            sort="service",
        )
        assert_matches_type(AsyncDefaultFlatPagination[TrafficPolicyProfile], traffic_policy_profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.traffic_policy_profiles.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        traffic_policy_profile = await response.parse()
        assert_matches_type(AsyncDefaultFlatPagination[TrafficPolicyProfile], traffic_policy_profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTelnyx) -> None:
        async with async_client.traffic_policy_profiles.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            traffic_policy_profile = await response.parse()
            assert_matches_type(
                AsyncDefaultFlatPagination[TrafficPolicyProfile], traffic_policy_profile, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncTelnyx) -> None:
        traffic_policy_profile = await async_client.traffic_policy_profiles.delete(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )
        assert_matches_type(TrafficPolicyProfileDeleteResponse, traffic_policy_profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.traffic_policy_profiles.with_raw_response.delete(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        traffic_policy_profile = await response.parse()
        assert_matches_type(TrafficPolicyProfileDeleteResponse, traffic_policy_profile, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncTelnyx) -> None:
        async with async_client.traffic_policy_profiles.with_streaming_response.delete(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            traffic_policy_profile = await response.parse()
            assert_matches_type(TrafficPolicyProfileDeleteResponse, traffic_policy_profile, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.traffic_policy_profiles.with_raw_response.delete(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_services(self, async_client: AsyncTelnyx) -> None:
        traffic_policy_profile = await async_client.traffic_policy_profiles.list_services()
        assert_matches_type(
            AsyncDefaultFlatPagination[TrafficPolicyProfileListServicesResponse],
            traffic_policy_profile,
            path=["response"],
        )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_services_with_all_params(self, async_client: AsyncTelnyx) -> None:
        traffic_policy_profile = await async_client.traffic_policy_profiles.list_services(
            filter_group="filter[group]",
            filter_name="filter[name]",
            page_number=1,
            page_size=1,
        )
        assert_matches_type(
            AsyncDefaultFlatPagination[TrafficPolicyProfileListServicesResponse],
            traffic_policy_profile,
            path=["response"],
        )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_list_services(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.traffic_policy_profiles.with_raw_response.list_services()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        traffic_policy_profile = await response.parse()
        assert_matches_type(
            AsyncDefaultFlatPagination[TrafficPolicyProfileListServicesResponse],
            traffic_policy_profile,
            path=["response"],
        )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_list_services(self, async_client: AsyncTelnyx) -> None:
        async with async_client.traffic_policy_profiles.with_streaming_response.list_services() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            traffic_policy_profile = await response.parse()
            assert_matches_type(
                AsyncDefaultFlatPagination[TrafficPolicyProfileListServicesResponse],
                traffic_policy_profile,
                path=["response"],
            )

        assert cast(Any, response.is_closed) is True
